
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>


// Asigna el valor de los parametros:
// npt    - numero de puntos a generar
// x0     - Extremo izquierdo del intervalo 
// x1     - Extremo derecho del intervalo 
void readParams(int argc, char **argv, int *nptx, int *npty, 
                double *x0, double *x1, double *y0, double *y1, char *cfile) {
    if(argc>1) *nptx = atoi(argv[1]);
    else       *nptx = 50;
    if(argc>2) *npty = atoi(argv[2]);
    else       *npty = 50;
    if(argc>3) *x0   = atof(argv[3]);
    else       *x0   = 0.0;
    if(argc>4) *x1   = atof(argv[4]);
    else       *x1   = 1.0;
    if(argc>5) *y0   = atof(argv[5]);
    else       *y0   = 0.0;
    if(argc>6) *y1   = atof(argv[6]);
    else       *y1   = 1.0;
    if(argc>7)  strcpy(cfile, argv[7]);
    else        strcpy(cfile, "salida.txt");
}


double fncf(double x, double y) {
    return(sin(x)*cos(y));
}


// Reserva memoria para una matriz de tama~no nr x nc
double **createMatrix(int nr, int nc) {
    int      i;
    double **mat;

    // Reservamos memoria
    mat    = (double **) malloc( (nr)*sizeof(double *));
    if(mat==NULL) return(NULL);
    mat[0] = (double *) malloc( nr*nc*sizeof(double));
    if(mat[0]==NULL) return(NULL);
    for(i=1; i<nr; ++i) 
        mat[i] = mat[i-1] + nc;
    return(mat);
}


// Libera la memoria del arreglo bidimensional
void freeMatrix(double **mat) {
    free(mat[0]);
    free(mat);
}



// Genera el archivo de texto que solo tiene los elementos del arreglo mat
void saveMatrixTxt(double **mat, int nr, int nc, char *cfile) {
    FILE   *fp=fopen(cfile, "wt");
    int     i, j;

    if(!fp) {
        printf("No se puede abrir el archivo\n");
        exit(0);
    }
    printf("Generando el archivo %s ...\n", cfile);

    for(i=0; i<nr; ++i) {
        for(j=0; j<nc; ++j) 
            fprintf(fp, "%.8e  ", mat[i][j]);
        fprintf(fp, "\n");
    }
    fclose(fp);
}


// Genera un archivo que tiene tres columnas, donde la primera tiene
// las abscisas x_j, la segunda los valores de las ordenadas y_j y la tercera
// los valores de la funci\'on en los puntos (x_j, y_i)
void saveMatrixGnuplot1(double **mat, int nptx, int npty, char *cfile) {
    FILE   *fp=fopen(cfile, "wt");
    int     i, j, k;

    if(!fp) {
        printf("No se puede abrir el archivo\n");
        exit(0);
    }
    printf("Generando el archivo %s ...\n", cfile);

    fprintf(fp, "#             x                y               z\n");
    for(i=0; i<nptx; ++i) {
        for(j=0; j<npty; ++j) {
            k  = j + i*npty;
            fprintf(fp, "%.8e   %.8e   %.8e\n", mat[k][0], mat[k][1], mat[k][2]);
        }
        fprintf(fp, "\n");
    }
    fclose(fp);
}


void generaTabla1(double x0, double x1, int nptx, double y0, double y1, int npty, 
                 char *cfile) {
    int      i, j, k;
    double   dincrx = (x1-x0)/(nptx-1), dincry = (y1-y0)/(npty-1), x, y;
    double **mat = createMatrix(nptx*npty, 3);

    for(i=0, x=x0; i<nptx; ++i, x+=dincrx) {
        for(j=0, y=y0; j<npty; ++j, y+=dincry)  {
            k         = j + i*npty;
            mat[k][0] = x;
            mat[k][1] = y;
            mat[k][2] = fncf(x, y);
        }
    }

    saveMatrixGnuplot1(mat, nptx, npty,  cfile);

    freeMatrix(mat);
}


void generaTabla2(double x0, double x1, int nptx, double y0, double y1, int npty, 
                 char *cfile) {
    int      i, j;
    double   dincrx = (x1-x0)/(nptx-1), dincry = (y1-y0)/(npty-1), x, y;
    double **mat = createMatrix(npty, nptx);

    for(i=0, y=y0; i<npty; ++i, y+=dincry)  {
        for(j=0, x=x0; j<nptx; ++j, x+=dincrx) {
            mat[i][j] = fncf(x, y);
        }
    }

    saveMatrixTxt(mat, npty, nptx, cfile);

    freeMatrix(mat);
}


int main(int argc, char **argv)      {
    int     nptx, npty;
    double  x0, x1, y0, y1;
    char    cfile[100], cfile2[100];

    readParams(argc, argv, &nptx, &npty, &x0, &x1, &y0, &y1, cfile);

    printf("\nPrograma que evalua una funcion de dos variables en puntos\n");
    printf("de una reticula definida por los intervalos [x0,x1] x [y0,y1].\n");
    printf("Los resultados se guardan en dos archivos: uno que tiene una  \n");
    printf("tabla en donde cada fila tiene los valores    \n");
    printf("x_j  y_i  f(x_j, y_i)    \n");
    printf("y otro que unicamente una matriz cuyo elemento en la posicion i,j\n");
    printf("es f(x_j, y_i) \n\n");

    printf(" Numero de puntos en x: %d\n", nptx);
    printf(" Numero de puntos en y: %d\n", npty);
    printf("Intervalo para evaluar: x[%f, %f]   y[%f, %f]\n\n", x0, x1, y0, y1);

    generaTabla1(x0, x1, nptx, y0, y1, npty, cfile);
    sprintf(cfile2, "sinCoord_%s", cfile);
    generaTabla2(x0, x1, nptx, y0, y1, npty, cfile2);

    return(0);
}
// Para compilar
// gcc -o generaDatosSuperficie  generaDatosSuperficie.c  -lm 

// Para ejecutar
// ./generaDatosSuperficie 40 40 -3 3 -3 3 tablaSuperficie.txt



