
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>


// Asigna el valor de los parametros:
// npt    - numero de puntos a generar
// x0     - Extremo izquierdo del intervalo 
// x1     - Extremo derecho del intervalo 
void readParams(int argc, char **argv, int *npt, double *x0, double *x1, 
                char *cfile) {
    if(argc>1) *npt = atoi(argv[1]);
    else       *npt = 100;
    if(argc>2) *x0  = atof(argv[2]);
    else       *x0  = 0.0;
    if(argc>3) *x1  = atof(argv[3]);
    else       *x1  = 1.0;
    if(argc>4)  strcpy(cfile, argv[4]);
    else        strcpy(cfile, "salida.txt");
}


double funcf1(double x) {
    return(4*sin(4*x));
}


double funcf2(double x) {
    return(0.5*x*x*x);
}


double funcf3(double x) {
    return(sqrt(fabs(x)));
}

// Reserva memoria para una matriz de tama~no nr x nc
double **createMatrix(int nr, int nc) {
    int      i;
    double **mat;

    // Reservamos memoria
    mat    = (double **) malloc( (nr)*sizeof(double *));
    if(mat==NULL) return(NULL);
    mat[0] = (double *) malloc( nr*nc*sizeof(double));
    if(mat[0]==NULL) return(NULL);
    for(i=1; i<nr; ++i) 
        mat[i] = mat[i-1] + nc;
    return(mat);
}


// Libera la memoria del arreglo bidimensional
void freeMatrix(double **mat) {
    free(mat[0]);
    free(mat);
}



void saveMatrixTxt(double **mat, int nr, int nc, char *cfile) {
    FILE   *fp=fopen(cfile, "wt");
    int     i, j;

    if(!fp) {
        printf("No se puede abrir el archivo\n");
        exit(0);
    }
    printf("Generando el archivo %s ...\n", cfile);

    for(i=0; i<nr; ++i) {
        for(j=0; j<nc; ++j) 
            fprintf(fp, "%.8e  ", mat[i][j]);
        fprintf(fp, "\n");
    }
    fclose(fp);
}


void generaTabla(double x0, double x1, int npt, char *cfile) {
    int      i;
    double   dincr = (x1-x0)/(npt-1), x;
    double **mat = createMatrix(npt, 4);

    for(i=0, x=x0; i<npt; ++i, x+=dincr) {
        mat[i][0] = x;
        mat[i][1] = funcf1(x);
        mat[i][2] = funcf2(x);
        mat[i][3] = funcf3(x);
    }

    saveMatrixTxt(mat, npt, 4, cfile);
    freeMatrix(mat);
}


int main(int argc, char **argv)      {
    int     npt;
    double  x0, x1;
    char    cfile[100];

    readParams(argc, argv, &npt, &x0, &x1, cfile);

    printf("\nPrograma para generar un archivo de texto que contiene una tabla\n");
    printf("en la que la primera columna tiene los puntos x_i en donde se     \n");
    printf("evaluan tres funciones. Las otras tienes columnas tienen los valores\n");
    printf("de  f1(x_i), f2(x_i) y  f3(x_i).   \n\n");

    printf("      Numero de puntos: %d\n", npt);
    printf("Intervalo para evaluar: [%f, %f]\n\n", x0, x1);

    generaTabla(x0, x1, npt, cfile);

    return(0);
}
// Para compilar
// gcc -o generaTabla  generaTabla.c  -lm 

// Para ejecutar
// ./generaTabla 100 -2 2 tabla.txt 




